# test.py Test program for IR remote control decoder
# Supports Pyboard, ESP32 and ESP8266

# Author: Peter Hinch
# Copyright Peter Hinch 2020-2022 Released under the MIT license

# Run this to characterise a remote.
import ustruct
from sys import platform
import time
import gc
from machine import Pin, freq
from olt_lib.ir_rx.print_error import print_error  # Optional print of error codes

# Import all implemented classes
from olt_lib.ir_rx.olt import LT_24, SONY_12, SONY_15, SONY_20

# Define pin according to platform
if platform == "pyboard":
    p = Pin("X3", Pin.IN)
elif platform == "esp8266":
    freq(160000000)
    p = Pin(13, Pin.IN)
elif platform == "esp32" or platform == "esp32_LoBo":
    p = Pin(23, Pin.IN)
elif platform == "rp2":
    p = Pin(16, Pin.IN)

def reverse(s):
    str = ""
    for i in s:
        str = i + str
    return str

# Bit reverse a 32 bit value
def rbit32(v):
    v = (v & 0x0000ffff) << 16 | (v & 0xffff0000) >> 16
    v = (v & 0x00ff00ff) << 8 | (v & 0xff00ff00) >> 8
    v = (v & 0x0f0f0f0f) << 4 | (v & 0xf0f0f0f0) >> 4
    v = (v & 0x33333333) << 2 | (v & 0xcccccccc) >> 2
    return (v & 0x55555555) << 1 | (v & 0xaaaaaaaa) >> 1

# User callback
def cb(byte1, byte2, byte3, packet):
#    print("TEST")
#    print(f"packet {packet}")
    print(f"byte1 0x{byte1:02x} byte2 0x{byte2:02x} byte3 0x{byte3:02x} packet 0x{packet:08x}")
    #packet.reverse()
    #print(packet)
    #reversed
    #print(f" XXX {ustruct.unpack(">I", packet)[0]}")
    #print(data)
    #ba = int.to_bits(packet, 1, 'big')
    #ba[0].reverse()
    test = bin(packet)
    print(test)
    test2 = reverse(test)
    print(test2)
    #test.__reversed__()
    test3 = rbit32(packet)
    print(f"{test3:08x}")
    #print(test)

#    print(f"test: {(int.to_bytes(packet, 4, 'big'))}")
    #ba = ba.reversed()
    
    #print(ba)
    #ba = int.to_bytes(packet, 4, 'little')
    #print(f"packet {ba:08x}")
            #ba = bytearray(val)
            #ba.reverse()
            #bytes = b'\xAA\x00\x00\x00'
    #packet = int.from_bytes(ba, byteorder='little', signed=False)
            #print(integer)
    #print(f"packet {packet:08x}")

def test(proto=0):
    classes = (LT_24, SONY_12, SONY_15, SONY_20)
    ir = classes[proto](p, cb)  # Instantiate receiver
    ir.error_function(print_error)  # Show debug information
    # ir.verbose = True
    # A real application would do something here...
    try:
        while True:
            print("running")
            time.sleep(5)
            gc.collect()
    except KeyboardInterrupt:
        ir.close()


# **** DISPLAY GREETING ****
s = """Test for IR receiver. Run:
from ir_rx.test import test
test() for LT 24 bit protocol,
test(1) for Sony SIRC 12 bit,
test(2) for Sony SIRC 15 bit,
test(3) for Sony SIRC 20 bit,

Hit ctrl-c to stop, then ctrl-d to soft reset."""

print(s)
